@@ -1,7 +1,13 @@
 import copy
+from typing import Union
+
+from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy
+
+_MD_Classes = Union[type[MultiDict[int]], type[CIMultiDict[int]]]
+_MDP_Classes = Union[type[MultiDictProxy[int]], type[CIMultiDictProxy[int]]]
 
 
-def test_copy(any_multidict_class):
+def test_copy(any_multidict_class: _MD_Classes) -> None:
     d = any_multidict_class()
     d["foo"] = 6
     d2 = d.copy()
@@ -10,7 +16,9 @@ def test_copy(any_multidict_class):
     assert d2["foo"] == 7
 
 
-def test_copy_proxy(any_multidict_class, any_multidict_proxy_class):
+def test_copy_proxy(
+    any_multidict_class: _MD_Classes, any_multidict_proxy_class: _MDP_Classes
+) -> None:
     d = any_multidict_class()
     d["foo"] = 6
     p = any_multidict_proxy_class(d)
@@ -21,7 +29,7 @@ def test_copy_proxy(any_multidict_class, any_multidict_proxy_class):
     assert d2["foo"] == 7
 
 
-def test_copy_std_copy(any_multidict_class):
+def test_copy_std_copy(any_multidict_class: _MD_Classes) -> None:
     d = any_multidict_class()
     d["foo"] = 6
     d2 = copy.copy(d)
@@ -30,7 +38,7 @@ def test_copy_std_copy(any_multidict_class):
     assert d2["foo"] == 7
 
 
-def test_ci_multidict_clone(any_multidict_class):
+def test_ci_multidict_clone(any_multidict_class: _MD_Classes) -> None:
     d = any_multidict_class(foo=6)
     d2 = any_multidict_class(d)
     d2["foo"] = 7
